/*
 * Copyright (c) 2002-2008 TeamDev Ltd. All rights reserved.
 *
 * Use is subject to license terms.
 *
 * The complete licence text can be found at
 * http://www.teamdev.com/winpack/license.jsf
 */
import teamdev.jxcapture.DrawableCapture;
import teamdev.jxcapture.ScreenObjectCapture;
import teamdev.jxcapture.controllers.ScreenObjectController;
import teamdev.jxcapture.controllers.SelectionController;
import teamdev.jxcapture.events.CaptureAdapter;
import teamdev.jxcapture.painters.ScreenObjectPainter;
import teamdev.jxcapture.painters.SelectionPainter;
import teamdev.jxcapture.toolkit.DrawableArea;
import teamdev.jxdesktop.PlatformContext;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;

/**
 * The example demonstrates how to set the specified popup menu for the capture operation.<p>
 *
 * <b>Note</b>: Available only for Windows platform.
 *
 * @author Ikryanov Vladimir
 */
public class CaptureWithPopupSample extends JxCaptureSample {

    public static void main(String[] args) {
        if (PlatformContext.isWindows()) {
            new CaptureWithPopupSample().run();
        } else {
            System.out.println("Unsupported Operation System. Please see JavaDocs.");
        }
    }

    protected void execute() {
        SelectionPainter painter = new ScreenObjectPainter();
        SelectionController controller = new ScreenObjectController(painter);
        final DrawableCapture captureOperation = new ScreenObjectCapture(controller);

        captureOperation.addCaptureListener(new CaptureAdapter() {
            public void cancel() {
                System.out.println("Cancel");
            }

            public void complete(BufferedImage image) {
                saveImageToHomeDirectory(image);
            }
        });

        // creating the popup menu
        JPopupMenu popupMenu = new JPopupMenu();
        popupMenu.add(new JMenuItem(new AbstractAction("Abort Capture") {
            public void actionPerformed(ActionEvent e) {
                captureOperation.cancel();
            }
        }));
        popupMenu.add(new JMenuItem(new AbstractAction("Finish Capture") {
            public void actionPerformed(ActionEvent e) {
                captureOperation.complete();
            }
        }));

        // setting the popup menu for the DrawableArea component
        DrawableArea drawableArea = captureOperation.getDrawableArea();
        drawableArea.setPopupMenu(popupMenu);

        // executing the capture operation
        captureOperation.execute();
    }

    protected String getFileName() {
        return "jxcaptureimage.png";
    }
}
